#include <EEPROM.h>

/*
Ben Nitkin / Fall 2012
RFID door lock

Overview: 
KEYS[] is a list of accepted keys. It's loaded from eeprom at boot, and can be modified 
 through add() and remove()
EEPROM stores 1kb, in integer format. I'm using longs for keys, 
 so some conversion is necessary. 
DATA1 and DATA0 are interrupt functions that grab the Wiegand data from the reader

add() and remove() add or remove the last key scanned from the list of allowed keys. 
 They're tripped by buttons.

load() and save() load keys from eeprom on boot, and save it to eeprom when KEYS is modified. 

EEPROM map
Memory site: Description of byte
0:       Number of keys stored in EEPROM
1-899:   Unallocated
900:     First 8 bits  (0-7)  of key 1
901:     Next 8 bits  (8-15)  of key 1
902:     Next 8 bits  (16-23) of key 1
903:     Final 8 bits (23-31) of key 1
904-907: Key 2 (same scheme as key 1)
908-911: Key 3
And so on. 

That gives us room for 300 keys (it's limited to 255 by memory address 1), while leaving 
most of the EEPROM reserved to do my bidding. 
*/

long watchdog = 0;

int SAVE = 7, //The pin to watch for buttons on
  OPEN = 8,     //Override to open door from inside.
  SERVO = 10,    //Servo signal wire.
  RED = 13,
  GREEN = 12,
  BEEP = 11,

//The EEPROM index to start storing keys at. 
//Lower numbers are reserved for other stuff.
  START = 900,
  UNLOCK_TIME = 5000; //Unlock time, in ms.

//Accepted keys.
unsigned long KEYS[255];

volatile unsigned long current = 0;
volatile boolean changed = false;

//Prints the key in the specified index of KEYS.
void printKey(int i) {
  Serial.print(KEYS[i], BIN); //The last 32 bits of a key, in binary. 
  //(The first 4 are part of the site code, so known.)
  Serial.print("  ");
  Serial.println(KEYS[i]); //Same thing, but in decimal.    
}

//Checks the last key against the list of loaded keys.
boolean check() {
  Serial.print("Checking key: ");
  for (int i = 0; i < 255; i++) {
    if (KEYS[i] == current) {
      Serial.println("yup.");
      return true;
    }
  }
  Serial.println("nope.");
  return false;
}

//Pulls some pin high for a moment to turn a servo and unlock a door.
void unlock() {
  Serial.println("Match!");
  analogWrite(SERVO, 255);
  delay(5000);
  analogWrite(SERVO, 0);  
}

void setup() {  
  //Initialize keys.
  for (int i=0;i<255;i++) KEYS[i] = 0ul;
  
  watchdog = millis();
  
  Serial.begin(57600);
  delay(10000);
  
  attachInterrupt(0, DATA0, FALLING);  // DATA0 (green) on HID reader, Pin 2 on Arduino
  attachInterrupt(1, DATA1, FALLING);  // DATA1 (white) on HID reader, Pin 3 on Arduino
  
  pinMode(SAVE, INPUT);
  digitalWrite(SAVE, HIGH); //pullup resistor

  pinMode(SERVO, OUTPUT);
  
  pinMode(RED, OUTPUT);  
  pinMode(GREEN, OUTPUT);  
  pinMode(BEEP, OUTPUT);
  disable();
  
  pinMode(OPEN, INPUT);
  digitalWrite(OPEN, HIGH); //pullup resistor

  load();
}

void loop() {
  delay(100);
  
  if (changed && dead()) {
    if (check()) {
      unlock();
    
      //Conversion issues: Serial.println("\n"+current) borks horribly.
      Serial.println();
      Serial.println(current);
    }
  }
    
  if (digitalRead(OPEN) == LOW) unlock();
  
  //Adds the first card read after button push.
  if (digitalRead(SAVE) == LOW) {
    //Beep for ack; beep twice when leaving mode.
    waitForCard();
    add();
  }
  changed = false;
}
